/*
 *   hxdcaldbUtil_gsogpt.c
 *         2009-10-20, created by Y.Terada
 *         2009-10-21, change name by Y.Terada
 *            Summary:   proc 1.x.x     gsoghf
 *                       proc 2.0 - 2.4 gsoght
 *                       proc 2.5 -     gsogpt
 *         2010-01-05, read VERSION by Y.Terada
 */
#include <stdlib.h>
#include <string.h>
#include "fitsio.h"
#include "hxdcaldbUtil.h"
#include "hxdFitsCommentUtil.h"
#include "atFunctions.h"
#include "aste_time.h"

static FILE*     hxd_gsogpt_ascii_fp;
static fitsfile* hxd_gsogpt_fits_fp;
static char tool_name[] = "hxdcaldbUtil_gsogpt";

#define GSOGPT_DEBUG 0

/* ===================================================================
 *   ASCII File I/O
 * ===================================================================*/
int hxdcaldbUtil_gsogpt_open_ASCII (char* gsogpt_ascii_fname){
  int status = HXDCALDBUTIL_STATUS_OK;
  hxd_gsogpt_ascii_fp = fopen(gsogpt_ascii_fname, "r");
  if (hxd_gsogpt_ascii_fp == NULL){
    fprintf(stderr, "%s: Cannot open ASCII file(%s)\n",
            tool_name, gsogpt_ascii_fname);
    status = HXDCALDBUTIL_STATUS_NG;
  }
  return status;
}

int hxdcaldbUtil_gsogpt_read_ASCII (GSO_GPT* gsogpt_data){
  int status = HXDCALDBUTIL_STATUS_OK;

  enum{
    GPT_PARAM_LONGTERM_A = 1,
    GPT_PARAM_LONGTERM_B, /* No.2 */
    GPT_PARAM_LONGTERM_C,
    GPT_PARAM_LONGTERM_D,
    GPT_PARAM_LONGTERM_E,
    GPT_PARAM_TEMP_A,     /* No.6 */
    GPT_PARAM_TEMP_B,
    GPT_PARAM_TSAA_A,     /* No.8, 32*8 members */
    GPT_PARAM_TSAA_B,
    GPT_PARAM_TSAA_C,
    GPT_PARAM_MOD_GA_1A,  /* No.11 */
    GPT_PARAM_MOD_GA_1B,
    GPT_PARAM_MOD_GA_1C,
    GPT_PARAM_MOD_GA_2A,  /* No.14 */
    GPT_PARAM_MOD_GA_2B,
    GPT_PARAM_MOD_GA_2C,
    GPT_PARAM_MOD_GA_3A,  /* No.17 */
    GPT_PARAM_MOD_GA_3B,
    GPT_PARAM_MOD_GA_3C,
    GPT_PARAM_MOD_C       /* No.20 */
  };

  /**------------ parameters in ASCII ------------**/
  double start_time;
  double end_time;
  int start_yyyymmdd; /* no exist in ASCII, generated by start_time */
  int start_hhmmss;   /* no exist in ASCII, generated by end_time */
  int parameter_id;    /* enum of GPT_PARAM_xxxx */
  double gsogpt_param[HXD_WEL_N_UNIT*2*HXDPI_NUM_SAA_PASS]; /* temporal field */
  /**-----------------------------------------------**/
  
  int N_row;

  AtTimeD attime;
  double prev_start_time = 0.0;
  double prev_end_time = 0.0;
  int member_id, member_max;

  double *write_p;

  if (gsogpt_data == NULL){
    fprintf(stderr, "hxdcaldbUtil: Invalid data pointer(gsogpt_data)\n");
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }
  
  if (hxd_gsogpt_ascii_fp == NULL){
    fprintf(stderr, "hxdcaldbUtil: Invalid file pointer(hxd_gsogh)\n");
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  N_row = -1;
  while(!feof(hxd_gsogpt_ascii_fp))   {
    fscanf(hxd_gsogpt_ascii_fp, "%lf %lf %d",
	   &start_time, &end_time, &parameter_id);

    if (parameter_id == GPT_PARAM_TSAA_A){
      member_max = HXD_WEL_N_UNIT * 2 * HXDPI_NUM_SAA_PASS;
    } else {
      member_max = HXD_WEL_N_UNIT * 2;
    }

    for(member_id=0;member_id<member_max; member_id++){
      fscanf(hxd_gsogpt_ascii_fp, " %lf", &gsogpt_param[member_id]);
    }

    /*** (1) check the format ***/
    if ( ((int)start_time) != ((int)prev_start_time) ) {
      /** (1-1) end of the time zone **/
      prev_start_time = start_time;
      prev_end_time   = end_time;
      N_row ++;
    } else {
      /** (1-2) same time zone: check **/
      if ( ((int)end_time) != ((int)prev_end_time) ){
        fprintf(stderr, 
                "%s:No.%d: same start time (%f), differnt end time (%f->%f)\n",
                tool_name, N_row, start_time, prev_end_time, end_time);
        status = HXDCALDBUTIL_STATUS_NG;
        return status;
      }
    }

    /*** (2) write the data into the structure ***/
    aste2attimeD(start_time,&attime);
    start_yyyymmdd = attime.yr*10000 + attime.mo*100 + attime.dy;
    start_hhmmss = attime.hr*10000 + attime.mn*100 + attime.sc;
    gsogpt_data->data[N_row].start_time     = start_time;
    gsogpt_data->data[N_row].end_time       = end_time;
    gsogpt_data->data[N_row].start_yyyymmdd = start_yyyymmdd;
    gsogpt_data->data[N_row].start_hhmmss   = start_hhmmss;

    switch (parameter_id){
    case GPT_PARAM_LONGTERM_A:
      write_p = gsogpt_data->data[N_row].longterm_a; break;
    case GPT_PARAM_LONGTERM_B:
      write_p = gsogpt_data->data[N_row].longterm_b; break;
    case GPT_PARAM_LONGTERM_C:
      write_p = gsogpt_data->data[N_row].longterm_c; break;
    case GPT_PARAM_LONGTERM_D:
      write_p = gsogpt_data->data[N_row].longterm_d; break;
    case GPT_PARAM_LONGTERM_E:
      write_p = gsogpt_data->data[N_row].longterm_e; break;
    case GPT_PARAM_TEMP_A:
      write_p = gsogpt_data->data[N_row].temp_a; break;
    case GPT_PARAM_TEMP_B:
      write_p = gsogpt_data->data[N_row].temp_b; break;
    case GPT_PARAM_TSAA_A:
      write_p = gsogpt_data->data[N_row].tsaa_a; break;
    case GPT_PARAM_TSAA_B:
      write_p = gsogpt_data->data[N_row].tsaa_b; break;
    case GPT_PARAM_TSAA_C:
      write_p = gsogpt_data->data[N_row].tsaa_c; break;
    case GPT_PARAM_MOD_GA_1A:
      write_p = gsogpt_data->data[N_row].mod_ga_1a; break;
    case GPT_PARAM_MOD_GA_1B:
      write_p = gsogpt_data->data[N_row].mod_ga_1b; break;
    case GPT_PARAM_MOD_GA_1C:
      write_p = gsogpt_data->data[N_row].mod_ga_1c; break;
    case GPT_PARAM_MOD_GA_2A:
      write_p = gsogpt_data->data[N_row].mod_ga_2a; break;
    case GPT_PARAM_MOD_GA_2B:
      write_p = gsogpt_data->data[N_row].mod_ga_2b; break;
    case GPT_PARAM_MOD_GA_2C:
      write_p = gsogpt_data->data[N_row].mod_ga_2c; break;
    case GPT_PARAM_MOD_GA_3A:
      write_p = gsogpt_data->data[N_row].mod_ga_3a; break;
    case GPT_PARAM_MOD_GA_3B:
      write_p = gsogpt_data->data[N_row].mod_ga_3b; break;
    case GPT_PARAM_MOD_GA_3C:
      write_p = gsogpt_data->data[N_row].mod_ga_3c; break;
    case GPT_PARAM_MOD_C:
      write_p = gsogpt_data->data[N_row].mod_c; break;
    }

    for(member_id=0;member_id<member_max; member_id++){
      write_p[member_id]= gsogpt_param[member_id];
    }

  } /** end of feof **/

  gsogpt_data->nrow = N_row+1;

  return status;
}

int hxdcaldbUtil_gsogpt_close_ASCII(void){
  int status = HXDCALDBUTIL_STATUS_OK;
  if (fclose(hxd_gsogpt_ascii_fp) ){
    fprintf(stderr, "%s: Cannot close ASCII file\n", tool_name);
    status = HXDCALDBUTIL_STATUS_NG;
  }
  return status;
}

/* ===================================================================
 *   Fits File  Create
 * ===================================================================*/
int hxdcaldbUtil_gsogpt_create_FITS(char* gsogpt_fits_fname){
  int status = HXDCALDBUTIL_STATUS_OK;
  int istat = 0;
  int bitpix = 8;
  long naxis2 = 1;
  long nrow=0;
  char detname[16] = "WELL_GSO";
  char code_name[18]   = "GAIN_PARAM_TABLE";
  char *extension_name = "GAIN_PARAM_TABLE";

  long naxes[1];
  int unit_id;

#define HXDPMT_GSOGAINHIST_TBL_N_KEYWORD 37

static char *ttype[HXDPMT_GSOGAINHIST_TBL_N_KEYWORD] = {
    "START_TIME", "YYYYMMDD", "HHMMSS", 
    "END_TIME", "MODEL_ID", 
    "W00_SLOW", "W00_FAST",
    "W01_SLOW", "W01_FAST",
    "W02_SLOW", "W02_FAST",
    "W03_SLOW", "W03_FAST",
    "W10_SLOW", "W10_FAST",
    "W11_SLOW", "W11_FAST",
    "W12_SLOW", "W12_FAST",
    "W13_SLOW", "W13_FAST",
    "W20_SLOW", "W20_FAST",
    "W21_SLOW", "W21_FAST",
    "W22_SLOW", "W22_FAST",
    "W23_SLOW", "W23_FAST",
    "W30_SLOW", "W30_FAST",
    "W31_SLOW", "W31_FAST",
    "W32_SLOW", "W32_FAST",
    "W33_SLOW", "W33_FAST"
};
static char *tform[HXDPMT_GSOGAINHIST_TBL_N_KEYWORD] = {
    "1D", "1J", "1J", 
    "1D", "1I", 
    /** Fit Results **/
    "32D", "32D", "32D", "32D", 
    "32D", "32D", "32D", "32D", 
    "32D", "32D", "32D", "32D", 
    "32D", "32D", "32D", "32D",
    "32D", "32D", "32D", "32D", 
    "32D", "32D", "32D", "32D", 
    "32D", "32D", "32D", "32D", 
    "32D", "32D", "32D", "32D"
};

 static char *tunit[HXDPMT_GSOGAINHIST_TBL_N_KEYWORD] = {
    "s", "", "",  
    "s", "",
    "","","","",
    "","","","",
    "","","","",
    "","","","",
    "","","","",
    "","","","",
    "","","","",
    "","","",""
  };
  char *keyword[HXDPMT_GSOGAINHIST_TBL_N_KEYWORD]={
    "TTYPE1  ", "TTYPE2  ", "TTYPE3  ", "TTYPE4  ", "TTYPE5  ", 
    "TTYPE6  ", "TTYPE7  ", "TTYPE8  ","TTYPE9  ", "TTYPE10  ", 
    "TTYPE11  ", "TTYPE12  ", "TTYPE13  ", "TTYPE14  ", "TTYPE15  ", 
    "TTYPE16  ", "TTYPE17  ", "TTYPE18  ", "TTYPE19  ", "TTYPE20  ",
    "TTYPE21  ", "TTYPE22  ", "TTYPE23  ", "TTYPE24  ", "TTYPE25  ", 
    "TTYPE26  ", "TTYPE27  ", "TTYPE28  ", "TTYPE29  ", "TTYPE30  ",
    "TTYPE31  ", "TTYPE32  ", "TTYPE33  ", "TTYPE34  ", "TTYPE35  ", 
    "TTYPE36  ", "TTYPE37  "
  };
  char *comment[HXDPMT_GSOGAINHIST_TBL_N_KEYWORD]={
    "Start time",  "date of START_TIME", "time of START_TIME",
    "End time", "Model ID",
    "Gain parameters of W00 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W01 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W02 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W03 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W10 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W11 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W12 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W13 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W20 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W21 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W22 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W23 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W30 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W31 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W32 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W33 PI_SLOW                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",

    "Gain parameters of W00 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W01 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W02 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W03 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W10 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W11 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W12 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W13 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W20 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W21 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W22 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W23 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W30 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W31 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W32 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
    "Gain parameters of W33 PI_FAST                                        01-05: long term coefficient (A,B,C,D,E)                              06-07: Temperature coefficient (A,B)                                  08-15: Parameter_A of T_SAA (8 passages)                              16-17: Parameter_B and _C T_SAA                                       18-20: Gaussian Parameter 1 (A,B,C).                                  21-23: Gaussian Parameter 2 (A,B,C).                                  24-26: Gaussian Parameter 3 (A,B,C).                                  27:    Modify param                                                   28-32: reserved",
  };

  int ncol = sizeof(ttype)/sizeof(*ttype);
  char *description = "Parameters of gain trend of HXD GSO";
  int  use_boundary = 1;
  char *ext_meanning = "";

  int hdu_num;

  if (fits_create_file(&hxd_gsogpt_fits_fp, gsogpt_fits_fname, &istat)) {
    fprintf(stderr, "%s:fits_create_file %s failed (%d)\n", tool_name, 
            gsogpt_fits_fname, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  } 

  if( fits_write_grphdr(hxd_gsogpt_fits_fp,
                        1, 8, 0, naxes, 0, 1, 1, &istat) ){
    fprintf(stderr, "%s:fits_write_grphdr failed (%d)\n", tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }


  for (hdu_num=0; hdu_num < HXDGSOGPT_PMT_N_LINE; hdu_num++){
    if( fits_create_hdu(hxd_gsogpt_fits_fp, &istat) ){
      fprintf(stderr, "%s:fits_create_hdu failed (%d)\n", tool_name, istat);
      status = HXDCALDBUTIL_STATUS_NG;
      return status;
    }
    if ( fits_write_btblhdr(hxd_gsogpt_fits_fp, naxis2, ncol, 
                            ttype, tform, tunit, 
                            extension_name, 0, &istat) ) {
      fprintf(stderr, "%s:fits_write_btblhdr failed (%d)\n", tool_name, istat);
      status = HXDCALDBUTIL_STATUS_NG;
      return status;
    }
    if (hxdcaldbUtil_write_fits_header(hxd_gsogpt_fits_fp, 
                                       detname, code_name, 
                                       description,
                                       gsogpt_fits_fname,
                                       use_boundary, ext_meanning)
        != HXDCALDBUTIL_STATUS_OK){
      fprintf(stderr, "%s: write_fits_header failed.\n", tool_name);
      status = HXDCALDBUTIL_STATUS_NG;
      return status;
    }
    
    hxdFitsComment_write(hxd_gsogpt_fits_fp,HXDPMT_GSOGAINHIST_TBL_N_KEYWORD,
                         keyword, comment, &istat);
    if (istat){
      fprintf(stderr, "%s: fits write comment failed.\n", tool_name);
      status = HXDCALDBUTIL_STATUS_NG;
      return status;
    }
  }

  
  if(fits_flush_file(hxd_gsogpt_fits_fp, &istat)){
    fprintf(stderr, "%s: fits_flush_file failed (%d)\n", tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  return status;
}

int hxdcaldbUtil_gsogpt_write_row_FITS(int irow, GSO_GPT_TBL* gpt_tbl){
  int status = HXDCALDBUTIL_STATUS_OK;
  int istat = 0;
  int colnum;
  long firstelem = 1;
  long nelements = 1;
  enum{
    GSOGPT_START_TIME = 1, 
      GSOGPT_YYYYMMDD, GSOGPT_HHMMSS, 
      GSOGPT_END_TIME, GSOGPT_MODEL_ID, 
      GSOGPT_W00_SLOW, GSOGPT_W00_FAST,
      GSOGPT_W01_SLOW, GSOGPT_W01_FAST,
      GSOGPT_W02_SLOW, GSOGPT_W02_FAST,
      GSOGPT_W03_SLOW, GSOGPT_W03_FAST,
      GSOGPT_W10_SLOW, GSOGPT_W10_FAST,
      GSOGPT_W11_SLOW, GSOGPT_W11_FAST,
      GSOGPT_W12_SLOW, GSOGPT_W12_FAST,
      GSOGPT_W13_SLOW, GSOGPT_W13_FAST,
      GSOGPT_W20_SLOW, GSOGPT_W20_FAST,
      GSOGPT_W21_SLOW, GSOGPT_W21_FAST,
      GSOGPT_W22_SLOW, GSOGPT_W22_FAST,
      GSOGPT_W23_SLOW, GSOGPT_W23_FAST,
      GSOGPT_W30_SLOW, GSOGPT_W30_FAST,
      GSOGPT_W31_SLOW, GSOGPT_W31_FAST,
      GSOGPT_W32_SLOW, GSOGPT_W32_FAST,
      GSOGPT_W33_SLOW, GSOGPT_W33_FAST
      };
  double param[HXDGSOGPT_N_PARAM];
  int unit_id;

  /*** START_TIME ***/
  colnum = GSOGPT_START_TIME;
  fits_write_col_dbl(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements, 
                    &gpt_tbl->start_time, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_write_col_dbl start_time failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** YYYYMMDD ***/
  colnum = GSOGPT_YYYYMMDD;
  fits_write_col_int(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements, 
                    &gpt_tbl->start_yyyymmdd, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_write_col_dbl start_yyyymmdd failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** HHMMSS ***/
  colnum = GSOGPT_HHMMSS;
  fits_write_col_int(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements, 
                    &gpt_tbl->start_hhmmss, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_write_col_dbl start_hhmmss failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** END_TIME ***/
  colnum = GSOGPT_END_TIME;
  fits_write_col_dbl(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements, 
                    &gpt_tbl->end_time, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_write_col_dbl end_time failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** MODEL_ID ***/
  colnum = GSOGPT_MODEL_ID;
  fits_write_col_int(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements, 
		     &gpt_tbl->model_id, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_write_col_dbl model_id failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** Parameters ***/
  for(unit_id=0; unit_id<HXD_WEL_N_UNIT; unit_id++){
    /*- slow -*/
    param[ 0] = gpt_tbl->longterm_a[unit_id+HXD_WEL_N_UNIT];
    param[ 1] = gpt_tbl->longterm_b[unit_id+HXD_WEL_N_UNIT];
    param[ 2] = gpt_tbl->longterm_c[unit_id+HXD_WEL_N_UNIT];
    param[ 3] = gpt_tbl->longterm_d[unit_id+HXD_WEL_N_UNIT];
    param[ 4] = gpt_tbl->longterm_e[unit_id+HXD_WEL_N_UNIT];
    param[ 5] = gpt_tbl->temp_a[unit_id+HXD_WEL_N_UNIT];
    param[ 6] = gpt_tbl->temp_b[unit_id+HXD_WEL_N_UNIT];
    param[ 7] = gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*0];
    param[ 8] = gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*1];
    param[ 9] = gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*2];
    param[10] = gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*3];
    param[11] = gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*4];
    param[12] = gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*5];
    param[13] = gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*6];
    param[14] = gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*7];
    param[15] = gpt_tbl->tsaa_b[unit_id+HXD_WEL_N_UNIT];
    param[16] = gpt_tbl->tsaa_c[unit_id+HXD_WEL_N_UNIT];
    param[17] = gpt_tbl->mod_ga_1a [unit_id+HXD_WEL_N_UNIT];
    param[18] = gpt_tbl->mod_ga_1b [unit_id+HXD_WEL_N_UNIT];
    param[19] = gpt_tbl->mod_ga_1c [unit_id+HXD_WEL_N_UNIT];
    param[20] = gpt_tbl->mod_ga_2a [unit_id+HXD_WEL_N_UNIT];
    param[21] = gpt_tbl->mod_ga_2b [unit_id+HXD_WEL_N_UNIT];
    param[22] = gpt_tbl->mod_ga_2c [unit_id+HXD_WEL_N_UNIT];
    param[23] = gpt_tbl->mod_ga_3a [unit_id+HXD_WEL_N_UNIT];
    param[24] = gpt_tbl->mod_ga_3b [unit_id+HXD_WEL_N_UNIT];
    param[25] = gpt_tbl->mod_ga_3c [unit_id+HXD_WEL_N_UNIT];
    param[26] = gpt_tbl->mod_c     [unit_id+HXD_WEL_N_UNIT];
    param[27] = 0; /** reserve **/
    param[28] = 0; /** reserve **/
    param[29] = 0; /** reserve **/
    param[30] = 0; /** reserve **/
    param[31] = 0; /** reserve **/

    colnum = GSOGPT_W00_SLOW + (unit_id * 2);
    nelements = HXDGSOGPT_N_PARAM;
    fits_write_col_dbl(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements,
		       param, &istat);
    if(istat){
      fprintf(stderr, "%s:fits_write_col_dbl W%d%d_SLOW failed (%d)\n",
	      tool_name, unit_id>>2, unit_id&0x03, istat);
      status = HXDCALDBUTIL_STATUS_NG;
      return status;
    }

    /*- fast -*/
    param[ 0] = gpt_tbl->longterm_a[unit_id];
    param[ 1] = gpt_tbl->longterm_b[unit_id];
    param[ 2] = gpt_tbl->longterm_c[unit_id];
    param[ 3] = gpt_tbl->longterm_d[unit_id];
    param[ 4] = gpt_tbl->longterm_e[unit_id];
    param[ 5] = gpt_tbl->temp_a[unit_id];
    param[ 6] = gpt_tbl->temp_b[unit_id];
    param[ 7] = gpt_tbl->tsaa_a[unit_id+32*0];
    param[ 8] = gpt_tbl->tsaa_a[unit_id+32*1];
    param[ 9] = gpt_tbl->tsaa_a[unit_id+32*2];
    param[10] = gpt_tbl->tsaa_a[unit_id+32*3];
    param[11] = gpt_tbl->tsaa_a[unit_id+32*4];
    param[12] = gpt_tbl->tsaa_a[unit_id+32*5];
    param[13] = gpt_tbl->tsaa_a[unit_id+32*6];
    param[14] = gpt_tbl->tsaa_a[unit_id+32*7];
    param[15] = gpt_tbl->tsaa_b[unit_id];
    param[16] = gpt_tbl->tsaa_c[unit_id];
    param[17] = gpt_tbl->mod_ga_1a [unit_id];
    param[18] = gpt_tbl->mod_ga_1b [unit_id];
    param[19] = gpt_tbl->mod_ga_1c [unit_id];
    param[20] = gpt_tbl->mod_ga_2a [unit_id];
    param[21] = gpt_tbl->mod_ga_2b [unit_id];
    param[22] = gpt_tbl->mod_ga_2c [unit_id];
    param[23] = gpt_tbl->mod_ga_3a [unit_id];
    param[24] = gpt_tbl->mod_ga_3b [unit_id];
    param[25] = gpt_tbl->mod_ga_3c [unit_id];
    param[26] = gpt_tbl->mod_c     [unit_id];
    param[27] = 0; /** reserve **/
    param[28] = 0; /** reserve **/
    param[29] = 0; /** reserve **/
    param[30] = 0; /** reserve **/
    param[31] = 0; /** reserve **/

    colnum = GSOGPT_W00_FAST + (unit_id * 2);
    nelements = HXDGSOGPT_N_PARAM;
    fits_write_col_dbl(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements,
		       param, &istat);
    if(istat){
      fprintf(stderr, "%s:fits_write_col_dbl W%d%d_FAST failed (%d)\n",
	      tool_name, unit_id>>2, unit_id&0x03, istat);
      status = HXDCALDBUTIL_STATUS_NG;
      return status;
    }
  }
  return status;
}

int hxdcaldbUtil_gsogpt_write_FITS(GSO_GPT* gpt_data){
  int status = HXDCALDBUTIL_STATUS_OK;
  int istat = 0;
 
  long naxis2;
  int hdutype;
  int irow;
  char detname[16] = "WELL_GSO"; /** pri header **/

  /** (1) update fits header **/
  naxis2= (long) gpt_data->nrow;
  if (fits_modify_key_lng(hxd_gsogpt_fits_fp, "NAXIS2", naxis2, "&", 
			  &istat)){
    fprintf(stderr, "%s:fits_update_key NAXIS2=%d failed HDU%d (%d)\n", 
	    tool_name, gpt_data->nrow, 1, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }
  
  /** (2) write data **/
  for(irow=1; irow<=gpt_data->nrow; irow++){
    status = 
      hxdcaldbUtil_gsogpt_write_row_FITS(irow, &gpt_data->data[irow-1]);
    if(status == HXDCALDBUTIL_STATUS_NG){
      fprintf(stderr, "%s: gsogpt write fits failed (row=%d)\n", 
	      tool_name, irow);
      return status;
    }
  } /** end of irow **/  
  
    /** (3) update fits header **/
  if (fits_write_date(hxd_gsogpt_fits_fp, &istat)) {
    fprintf(stderr,"%s: fits_write_date failed (%d)\n", tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }
  
  if (fits_write_chksum(hxd_gsogpt_fits_fp, &istat)) {
    fprintf(stderr, "%s:  fits_write_chksum failed (%d)\n", tool_name, 
	    istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }
  
  if(fits_flush_file(hxd_gsogpt_fits_fp, &istat)){
    fprintf(stderr, "%s: fits_flush_file failed (%d)\n", tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /******** Add Primary Header keywords ********/
  if (hxdcaldbUtil_write_fits_priheader(hxd_gsogpt_fits_fp, detname)
      != HXDCALDBUTIL_STATUS_OK){
    fprintf(stderr, "%s: write_fits_header failed.\n", tool_name);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  return status;
}

/* ===================================================================
 *   Fits File  Read
 * ===================================================================*/
int hxdcaldbUtil_gsogpt_open_FITS(char* gsogpt_fits_fname){
  int status = HXDCALDBUTIL_STATUS_OK;
  int istat  = 0;
  char gsoghf_code_name[16]   = "GAIN_HISTORY";
  char gsoght_code_name[16]   = "GAIN_HIST_PARAM";
  char gsogpt_code_name[18]   = "GAIN_PARAM_TABLE";
  int hdutype;
  char comment[80];
  char code_name[32];

  /** open FITS file **/
  if (fits_open_file(&hxd_gsogpt_fits_fp, gsogpt_fits_fname, READONLY, 
		     &istat)) {
    fprintf(stderr, "%s:fits_open_file %s failed (%d)\n", tool_name, 
            gsogpt_fits_fname, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /** check code name (to avoid gsoght or gsoghf)**/
  if(fits_movabs_hdu(hxd_gsogpt_fits_fp, 2, &hdutype, &istat)){
    fprintf(stderr, "%s:fits_movabs_hdu 2 failed (%d)\n", 
	    tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  } else if(fits_read_key_str(hxd_gsogpt_fits_fp, "CCNM0001", 
		       code_name, comment, &istat) ){
    fprintf(stderr, "%s:fits_read_key_str CCNM0001 failed (%d)\n", 
	    tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  if(GSOGPT_DEBUG) 
    fprintf(stdout,"%s: code_name = %s\n", tool_name, code_name);

  if( strcmp(code_name, gsogpt_code_name) ){
    fprintf(stderr, 
	    "%s: Invalid caldb file (ae_hxd_gsogpt.fits is requested).\n",
	    tool_name);
    if( ! strcmp(code_name, gsoghf_code_name) ){
      fprintf(stderr, "%s: The file format is old (ae_hxd_gsoghf.fits).\n");
    } else  if( ! strcmp(code_name, gsoght_code_name) ){
      fprintf(stderr, "%s: The file format is old (ae_hxd_gsoght.fits).\n");
    } else {
      fprintf(stderr, "%s: Unknown file format (code_name = %s)", code_name);
    }
    fprintf(stderr, "%s: Quit the process.\n");
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  } else {
    if(GSOGPT_DEBUG) 
      fprintf(stdout,"%s: this is gsogpt file, continue\n", tool_name);
  }

  return status;
}

int hxdcaldbUtil_gsogpt_read_VERSION_FITS(int *version){
  int status = HXDCALDBUTIL_STATUS_OK;
  int istat  = 0;
  int hdutype;
  char comment[80];
  long gsogpt_v;

   /** (1) move to the HDU **/
  fits_movabs_hdu(hxd_gsogpt_fits_fp, 2, &hdutype, &istat);
  if (istat){
    fprintf(stderr, "%s: fits_movabs_hdu failed To HDU %d (status=%d)\n",
	    tool_name, 2, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }
  
   /** (2) read VERSION keyword **/
  fits_read_key_lng(hxd_gsogpt_fits_fp, "VERSION", 
		    &gsogpt_v, comment, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_read_key VERSION failed (%d)\n", 
	    tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  *version = gsogpt_v;

  return status;
}


int hxdcaldbUtil_gsogpt_read_row_FITS(int irow, GSO_GPT_TBL* gpt_tbl){
  int status = HXDCALDBUTIL_STATUS_OK;
  int istat  = 0;

  int colnum, anynul;
  long firstelem = 1;
  long nelements = 1;
  int    int_nulval = 0;
  double dbl_nulval = 0.0;
  enum{
    GSOGPT_START_TIME = 1, 
      GSOGPT_YYYYMMDD, GSOGPT_HHMMSS, 
      GSOGPT_END_TIME, GSOGPT_MODEL_ID, 
      GSOGPT_W00_SLOW, GSOGPT_W00_FAST,
      GSOGPT_W01_SLOW, GSOGPT_W01_FAST,
      GSOGPT_W02_SLOW, GSOGPT_W02_FAST,
      GSOGPT_W03_SLOW, GSOGPT_W03_FAST,
      GSOGPT_W10_SLOW, GSOGPT_W10_FAST,
      GSOGPT_W11_SLOW, GSOGPT_W11_FAST,
      GSOGPT_W12_SLOW, GSOGPT_W12_FAST,
      GSOGPT_W13_SLOW, GSOGPT_W13_FAST,
      GSOGPT_W20_SLOW, GSOGPT_W20_FAST,
      GSOGPT_W21_SLOW, GSOGPT_W21_FAST,
      GSOGPT_W22_SLOW, GSOGPT_W22_FAST,
      GSOGPT_W23_SLOW, GSOGPT_W23_FAST,
      GSOGPT_W30_SLOW, GSOGPT_W30_FAST,
      GSOGPT_W31_SLOW, GSOGPT_W31_FAST,
      GSOGPT_W32_SLOW, GSOGPT_W32_FAST,
      GSOGPT_W33_SLOW, GSOGPT_W33_FAST
      };
  double param[HXDGSOGPT_N_PARAM];
  int unit_id;

  /*** START_TIME ***/
  colnum = GSOGPT_START_TIME;
  fits_read_col_dbl(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements,
		    dbl_nulval, &gpt_tbl->start_time, &anynul, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_read_col_dbl start_time failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** YYYYMMDD ***/
  colnum = GSOGPT_YYYYMMDD;
  fits_read_col_int(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements,
		    int_nulval, &gpt_tbl->start_yyyymmdd, &anynul, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_read_col_dbl start_yyyymmdd failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** HHMMSS ***/
  colnum = GSOGPT_HHMMSS;
  fits_read_col_int(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements,
		    dbl_nulval, &gpt_tbl->start_hhmmss, &anynul, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_read_col_dbl start_hhmmss failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** END_TIME ***/
  colnum = GSOGPT_END_TIME;
  fits_read_col_dbl(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements,
		    dbl_nulval, &gpt_tbl->end_time, &anynul, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_read_col_dbl end_time failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** MODEL_ID ***/
  colnum = GSOGPT_MODEL_ID;
  fits_read_col_int(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements,
		    dbl_nulval, &gpt_tbl->model_id, &anynul, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_read_col_dbl model_id failed (%d)\n",
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  /*** Parameters ***/
  for(unit_id=0; unit_id<HXD_WEL_N_UNIT; unit_id++){
    /*- slow -*/
    colnum = GSOGPT_W00_SLOW + (unit_id * 2);
    nelements = HXDGSOGPT_N_PARAM;
    fits_read_col_dbl(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements,
		      dbl_nulval, param, &anynul, &istat);
    if(istat){
      fprintf(stderr, "%s:fits_read_col_dbl W%d%d_SLOW failed (%d)\n",
              tool_name, unit_id>>2, unit_id&0x03, istat);
      status = HXDCALDBUTIL_STATUS_NG;
      return status;
    }
    gpt_tbl->longterm_a[unit_id+HXD_WEL_N_UNIT] =     param[ 0];
    gpt_tbl->longterm_b[unit_id+HXD_WEL_N_UNIT] =     param[ 1];
    gpt_tbl->longterm_c[unit_id+HXD_WEL_N_UNIT] =     param[ 2];
    gpt_tbl->longterm_d[unit_id+HXD_WEL_N_UNIT] =     param[ 3];
    gpt_tbl->longterm_e[unit_id+HXD_WEL_N_UNIT] =     param[ 4];
    gpt_tbl->temp_a[unit_id+HXD_WEL_N_UNIT]     =     param[ 5];
    gpt_tbl->temp_b[unit_id+HXD_WEL_N_UNIT]     =     param[ 6];
    gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*0]=     param[ 7];
    gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*1]=     param[ 8];
    gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*2]=     param[ 9];
    gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*3]=     param[10];
    gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*4]=     param[11];
    gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*5]=     param[12];
    gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*6]=     param[13];
    gpt_tbl->tsaa_a[unit_id+HXD_WEL_N_UNIT+32*7]=     param[14];
    gpt_tbl->tsaa_b[unit_id+HXD_WEL_N_UNIT]     =     param[15];
    gpt_tbl->tsaa_c[unit_id+HXD_WEL_N_UNIT]     =     param[16];
    gpt_tbl->mod_ga_1a [unit_id+HXD_WEL_N_UNIT] =     param[17];
    gpt_tbl->mod_ga_1b [unit_id+HXD_WEL_N_UNIT] =     param[18];
    gpt_tbl->mod_ga_1c [unit_id+HXD_WEL_N_UNIT] =     param[19];
    gpt_tbl->mod_ga_2a [unit_id+HXD_WEL_N_UNIT] =     param[20];
    gpt_tbl->mod_ga_2b [unit_id+HXD_WEL_N_UNIT] =     param[21];
    gpt_tbl->mod_ga_2c [unit_id+HXD_WEL_N_UNIT] =     param[22];
    gpt_tbl->mod_ga_3a [unit_id+HXD_WEL_N_UNIT] =     param[23];
    gpt_tbl->mod_ga_3b [unit_id+HXD_WEL_N_UNIT] =     param[24];
    gpt_tbl->mod_ga_3c [unit_id+HXD_WEL_N_UNIT] =     param[25];
    gpt_tbl->mod_c     [unit_id+HXD_WEL_N_UNIT] =     param[26];
    /** param[27-31] are reserved space**/

    /*- fast -*/
    colnum = GSOGPT_W00_FAST + (unit_id * 2);
    nelements = HXDGSOGPT_N_PARAM;
    fits_read_col_dbl(hxd_gsogpt_fits_fp, colnum, irow, firstelem, nelements,
		      dbl_nulval, param, &anynul, &istat);
    if(istat){
      fprintf(stderr, "%s:fits_read_col_dbl W%d%d_FAST failed (%d)\n",
              tool_name, unit_id>>2, unit_id&0x03, istat);
      status = HXDCALDBUTIL_STATUS_NG;
      return status;
    }
    gpt_tbl->longterm_a[unit_id] =     param[ 0];
    gpt_tbl->longterm_b[unit_id] =     param[ 1];
    gpt_tbl->longterm_c[unit_id] =     param[ 2];
    gpt_tbl->longterm_d[unit_id] =     param[ 3];
    gpt_tbl->longterm_e[unit_id] =     param[ 4];
    gpt_tbl->temp_a[unit_id]     =     param[ 5];
    gpt_tbl->temp_b[unit_id]     =     param[ 6];
    gpt_tbl->tsaa_a[unit_id+32*0]=     param[ 7];
    gpt_tbl->tsaa_a[unit_id+32*1]=     param[ 8];
    gpt_tbl->tsaa_a[unit_id+32*2]=     param[ 9];
    gpt_tbl->tsaa_a[unit_id+32*3]=     param[10];
    gpt_tbl->tsaa_a[unit_id+32*4]=     param[11];
    gpt_tbl->tsaa_a[unit_id+32*5]=     param[12];
    gpt_tbl->tsaa_a[unit_id+32*6]=     param[13];
    gpt_tbl->tsaa_a[unit_id+32*7]=     param[14];
    gpt_tbl->tsaa_b[unit_id]     =     param[15];
    gpt_tbl->tsaa_c[unit_id]     =     param[16];
    gpt_tbl->mod_ga_1a [unit_id] =     param[17];
    gpt_tbl->mod_ga_1b [unit_id] =     param[18];
    gpt_tbl->mod_ga_1c [unit_id] =     param[19];
    gpt_tbl->mod_ga_2a [unit_id] =     param[20];
    gpt_tbl->mod_ga_2b [unit_id] =     param[21];
    gpt_tbl->mod_ga_2c [unit_id] =     param[22];
    gpt_tbl->mod_ga_3a [unit_id] =     param[23];
    gpt_tbl->mod_ga_3b [unit_id] =     param[24];
    gpt_tbl->mod_ga_3c [unit_id] =     param[25];
    gpt_tbl->mod_c     [unit_id] =     param[26];
    /** param[27-31] are reserved space**/
    /** param[4-8] are reserved space**/
  }

  return status;
}


int hxdcaldbUtil_gsogpt_read_FITS(double tstart, double tstop,
				  GSO_GPT* gpt_data, int numrow){
  int status = HXDCALDBUTIL_STATUS_OK;
  int istat  = 0;
  int hdutype;
  char comment[80];
  /** for row search **/
  double gpt_start_time;
  int startrow, stoprow;
  /** read start time **/
  int colnum,anynul;
  long irow, nrow;
  long firstelem = 1;
  long nelements = 1;
  enum {
    GSOGPT_START_TIME=1
  };
  double  double_nulval = 0.0;
  int i;

   /** (1) move to the HDU **/
  fits_movabs_hdu(hxd_gsogpt_fits_fp, 2, &hdutype, &istat);
  if (istat){
    fprintf(stderr, "%s: fits_movabs_hdu failed To HDU %d (status=%d)\n",
	    tool_name, 2, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }
  
  /** (2) read number of rows **/
  fits_read_key_lng(hxd_gsogpt_fits_fp, "NAXIS2", &nrow, comment, &istat);
  if(istat){
    fprintf(stderr, "%s:fits_read_key NAXIS2 failed (%d)\n", 
	    tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }

  numrow = nrow;
  
  /** (3) search row region, using tstart and tstop (only hdu_num==0) **/
  /*******    This  section is skipped in case of gpt *****/
  
  /** (4) read the data **/
  for(irow=1;irow<=nrow;irow++){
    status = 
      hxdcaldbUtil_gsogpt_read_row_FITS(irow, &gpt_data->data[irow-1]);
    if(status == HXDCALDBUTIL_STATUS_NG){
      fprintf(stderr, "%s: gsogpt read fits failed (row=%d)\n", 
	      tool_name, irow);
      return status;
    }
  } /** end of irow **/

  gpt_data->nrow = nrow;
  
  return status;
}

int hxdcaldbUtil_gsogpt_close_FITS(void){
  int status = HXDCALDBUTIL_STATUS_OK;
  int istat = 0;
  
  if(fits_close_file(hxd_gsogpt_fits_fp, &istat)){
    fprintf(stderr, "%s:fits_close_file failed (%d)\n", 
            tool_name, istat);
    status = HXDCALDBUTIL_STATUS_NG;
    return status;
  }
  return status;
}

/* ========================= EOF =========================== */
